/* Fo
 * expresion-parser.c: XSL expression parser
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: expression-parser.c,v 1.1.1.1 2003/02/13 00:51:16 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include "fo-expr-eval.h"

/**
 * xslEvalExpression2:
 * @str:               String for the expression
 * @property_name:     Name of the property being evaluated
 * @fo_context:        #FoContext from which to get inherited values
 * @resolve_enum_func: #FoResolveEnumFunc function used to resolve enum tokens
 * @resolve_percent_func: #FoResolvePercentFunc function used to resolve percentages
 * @fo_node:           Current node in the Fo tree
 * @current_font_size: Current font-size property
 * @error:             #GError
 * 
 * Evaluate @str as an XSL FO expression w.r.t. @current_font_size and
 * using @fo_context for inherited values and @func for resolution of
 * enumeration tokens.
 *
 * If an error occurred, @error is set.
 *
 * Calling function must free the result.
 * 
 * Return value: #FoDatatype that is the result of the expression, or NULL.
 **/
FoDatatype*
xslEvalExpression2 (const xmlChar *str,
		    const xmlChar *property_name,
		    const FoContext *fo_context,
		    const FoResolveEnumFunc resolve_enum_func,
		    const FoResolvePercentFunc resolve_percent_func,
		    const FoFo *fo_node,
		    const FoProperty *current_font_size,
		    GError **error)
{
  static FoExprEnvList *env_list = NULL;

  if (env_list == NULL)
    {
      env_list = fo_expr_env_list_new ();
    }

  return fo_expr_eval (str,
		       property_name,
		       resolve_enum_func,
		       resolve_percent_func,
		       current_font_size,
		       fo_node,
		       fo_context,
		       env_list,
		       error);
}

/**
 * xslEvalExpression:
 * @str:               String for the expression
 * @property_name:     Name of the property being evaluated
 * @fo_context:        #FoContext from which to get inherited values
 * @resolve_enum_func: #FoResolveEnumFunc function used to resolve enum tokens
 * @resolve_percent_func: #FoResolvePercentFunc function used to resolve percentages
 * @fo_node:           Current node in the Fo tree
 * @current_font_size: Current font-size property
 * 
 * Evaluate @str as an XSL FO expression w.r.t. @current_font_size and
 * using @fo_context for inherited values and @func for resolution of
 * enumeration tokens.
 * 
 * Calling function must free the result.
 * 
 * Return value: #FoDatatype that is the result of the expression
 **/
FoDatatype*
xslEvalExpression (const xmlChar *str,
		   const xmlChar *property_name,
		   const FoContext *fo_context,
		   const FoResolveEnumFunc resolve_enum_func,
		   const FoResolvePercentFunc resolve_percent_func,
		   const FoFo *fo_node,
		   const FoProperty *current_font_size)
{
  FoDatatype *result = NULL;
  GError *error = NULL;

  result = xslEvalExpression2 (str,
			       property_name,
			       fo_context,
			       resolve_enum_func,
			       resolve_percent_func,
			       fo_node,
			       current_font_size,
			       &error);
  
  if (error != NULL)
    {
      fo_object_log_warning (FO_OBJECT (fo_node),
			     &error);
      g_error_free (error);
    }

  return result;
}
