/* Fo
 * fo-marker-parent.c: Formatting object that is parent of possible fo:marker
 *
 * Copyright (C) 2003 Sun Microsystems
 *
 * $Id: fo-marker-parent.c,v 1.1 2003/03/07 17:54:52 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include "fo-table.h"
#include "fo-marker-parent.h"
#include "fo-marker-parent-private.h"

enum {
  PROP_0
};

static void fo_marker_parent_base_init  (FoMarkerParentClass *klass);
static void fo_marker_parent_class_init  (FoMarkerParentClass *klass);
static void fo_marker_parent_finalize    (GObject           *object);
static void fo_marker_parent_debug_dump_properties (FoFo *fo, gint depth);

static gpointer parent_class;

GType
fo_marker_parent_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoMarkerParentClass),
        (GBaseInitFunc) fo_marker_parent_base_init,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) fo_marker_parent_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoMarkerParent),
        0,              /* n_preallocs */
        NULL,		/* instance_init */
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_FO,
                                            "FoMarkerParent",
                                            &object_info,
					    G_TYPE_FLAG_ABSTRACT);
    }

  return object_type;
}

/**
 * fo_marker_parent_base_init:
 * @klass: #FoMarkerParentClass base class object to initialise.
 * 
 * Implements #GBaseInitFunc for #FoMarkerParent.
 **/
void
fo_marker_parent_base_init (FoMarkerParentClass *klass)
{
  FoFoClass *fo_fo_class = FO_FO_CLASS (klass);

}

/**
 * fo_marker_parent_class_init:
 * @klass: #FoMarkerParentClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoMarkerParent.
 **/
void
fo_marker_parent_class_init (FoMarkerParentClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoFoClass *fo_fo_class = FO_FO_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_marker_parent_finalize;

  fo_fo_class->debug_dump_properties = fo_marker_parent_debug_dump_properties;
  fo_fo_class->update_from_context = fo_marker_parent_update_from_context;
}

static void
fo_marker_parent_finalize (GObject *object)
{
  FoMarkerParent *fo_marker_parent;

  fo_marker_parent = FO_MARKER_PARENT (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_marker_parent_new:
 * 
 * Creates a new #FoMarkerParent initialized to default value.
 * 
 * Return value: the new #FoMarkerParent.
 **/
FoFo*
fo_marker_parent_new (void)
{
  return FO_FO (g_object_new (fo_marker_parent_get_type (),
			      NULL));
}

void
fo_marker_parent_update_from_context (FoFo *fo, FoContext *context)
{
  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_MARKER_PARENT (fo));

  FO_FO_CLASS (parent_class)->update_from_context (fo, context);
}

void
fo_marker_parent_debug_dump_properties (FoFo *fo, gint depth)
{
  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_MARKER_PARENT (fo));

  FO_FO_CLASS (parent_class)->debug_dump_properties (fo, depth + 1);
}
