/* Fo
 * fo-doc.h: Wrapper for libfo output document
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-doc.h,v 1.4 2003/07/02 10:47:09 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#ifndef __FO_DOC_H__
#define __FO_DOC_H__

#include <datatype/fo-datatype.h>
#include <libfo/fo-utils.h>
#include <libfo/fo-libfo-context.h>
#include <libfo/fo-font-desc.h>

typedef struct _FoDoc      FoDoc;
typedef struct _FoDocClass FoDocClass;

#include <libfo/fo-layout.h>
#include <area/fo-area.h>

G_BEGIN_DECLS

#define FO_DOC_ERROR fo_doc_error_quark ()
GQuark fo_doc_error_quark (void);

typedef enum
{
  FO_DOC_ERROR_FAILED,
  FO_DOC_ERROR_OPEN_FAILED,   /* Cannot open output document */
  FO_DOC_ERROR_LAST
} FoDocError;

extern const char *fo_doc_error_messages[FO_DOC_ERROR_LAST];

typedef enum
{
  FO_DOC_LINE_CAP_INVALID,
  FO_DOC_LINE_CAP_BUTT,
  FO_DOC_LINE_CAP_ROUND,
  FO_DOC_LINE_CAP_SQUARE,
  FO_DOC_LINE_CAP_LIMIT
} FoDocLineCap;

typedef enum
{
  FO_DOC_LINE_JOIN_INVALID,
  FO_DOC_LINE_JOIN_MITER,
  FO_DOC_LINE_JOIN_ROUND,
  FO_DOC_LINE_JOIN_BEVEL,
  FO_DOC_LINE_JOIN_LIMIT
} FoDocLineJoin;

#define FO_TYPE_DOC              (fo_doc_get_type ())
#define FO_DOC(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), FO_TYPE_DOC, FoDoc))
#define FO_DOC_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), FO_TYPE_DOC, FoDocClass))
#define FO_IS_DOC(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), FO_TYPE_DOC))
#define FO_IS_DOC_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), FO_TYPE_DOC))
#define FO_DOC_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), FO_TYPE_DOC, FoDocClass))


GType         fo_doc_get_type (void);
FoDoc *       fo_doc_new (void);
FoDoc *       fo_doc_new_from_type (const gchar *type);

void          fo_doc_open_file (FoDoc       *fo_doc,
				const gchar *filename,
				GError     **error);

FoLayout *    fo_doc_get_new_layout (FoDoc *fo_doc);

const gchar * fo_doc_get_language (FoDoc *fo_doc);
void          fo_doc_set_language (FoDoc *fo_doc,
				   const gchar    *language);
FoEnumAreaDirection fo_doc_get_base_dir (FoDoc      *fo_doc);
void                fo_doc_set_base_dir (FoDoc      *fo_doc,
					 FoEnumAreaDirection  base_dir);

void          fo_doc_begin_page     (FoDoc        *fo_doc,
				     gfloat        width,
				     gfloat        height);
void          fo_doc_end_page       (FoDoc        *fo_doc);
FoDatatype *  fo_doc_get_fill_color (FoDoc        *fo_doc);
void          fo_doc_set_fill_color (FoDoc        *fo_doc,
				     FoDatatype   *color);
FoDatatype *  fo_doc_get_stroke_color (FoDoc        *fo_doc);
void          fo_doc_set_stroke_color (FoDoc        *fo_doc,
				       FoDatatype   *color);
FoDocLineCap  fo_doc_get_line_cap   (FoDoc        *fo_doc);
void          fo_doc_set_line_cap   (FoDoc        *fo_doc,
				     FoDocLineCap  line_cap);
FoDocLineJoin fo_doc_get_line_join  (FoDoc        *fo_doc);
void          fo_doc_set_line_join  (FoDoc        *fo_doc,
				     FoDocLineJoin line_join);
gfloat        fo_doc_get_line_width (FoDoc        *fo_doc);
void          fo_doc_set_line_width (FoDoc        *fo_doc,
				     gfloat        line_width);
void          fo_doc_set_dash       (FoDoc        *fo_doc,
				     gfloat        b,
				     gfloat        w);
void          fo_doc_translate      (FoDoc        *fo_doc,
				     gfloat        x,
				     gfloat        y);
void          fo_doc_clip           (FoDoc        *fo_doc);
void          fo_doc_save           (FoDoc        *fo_doc);
void          fo_doc_restore        (FoDoc        *fo_doc);
void          fo_doc_line_to        (FoDoc        *fo_doc,
				     gfloat        x,
				     gfloat        y);
void          fo_doc_move_to        (FoDoc        *fo_doc,
				     gfloat        x,
				     gfloat        y);
void          fo_doc_line_stroked   (FoDoc        *fo_doc,
				     gfloat        x0,
				     gfloat        y0,
				     gfloat        x1,
				     gfloat        y1);
void          fo_doc_rect_stroked   (FoDoc        *fo_doc,
				     gfloat        x,
				     gfloat        y,
				     gfloat        width,
				     gfloat        height);
void          fo_doc_rect_filled    (FoDoc        *fo_doc,
				     gfloat        x,
				     gfloat        y,
				     gfloat        width,
				     gfloat        height);
void          fo_doc_fill           (FoDoc        *fo_doc);
void          fo_doc_stroke         (FoDoc        *fo_doc);
void          fo_doc_fill_stroke    (FoDoc        *fo_doc);

void          fo_doc_place_image     (FoDoc        *fo_doc,
				      gint          image,
				      gfloat        x,
				      gfloat        y,
				      gfloat        scale);
gint          fo_doc_open_image_file (FoDoc        *fo_doc,
				      const char   *imagetype,
				      const char   *filename,
				      const char   *stringparam,
				      gint          intparam);
void          fo_doc_close_image     (FoDoc        *fo_doc,
				      gint          image);

FoFontDesc *  fo_doc_get_font_desc  (FoDoc *fo_doc);

void 	      fo_doc_render_layout_lines (FoDoc   *fo_doc,
					  FoArea  *area_layout,
					  gfloat   x,
					  gfloat   y);
void 	      fo_doc_render_layout       (FoDoc   *fo_doc,
					  FoArea  *area_layout,
					  gfloat   x,
					  gfloat   y);

G_END_DECLS

#endif /* !__FO_DOC_H__ */
