/* Fo
 * fo-property-background-color.c: 'background-color' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-background-color.c,v 1.5 2003/04/15 12:01:35 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-background-color.h"

/* Inherited: FALSE */
/* Shorthand: FALSE */
/* background-color */
/* <color> | transparent | inherit */
GType
fo_enum_background_color_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_BACKGROUND_COLOR_TRANSPARENT, "FO_ENUM_BACKGROUND_COLOR_TRANSPARENT", "transparent" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumBackgroundColor", values);
    }
  return etype;
}

struct _FoPropertyBackgroundColor
{
  FoProperty parent_instance;
};

struct _FoPropertyBackgroundColorClass
{
  FoPropertyClass parent_class;
};

static void fo_property_background_color_init         (FoPropertyBackgroundColor      *property_background_color);
static void fo_property_background_color_base_init   (FoPropertyBackgroundColorClass *klass);
static void fo_property_background_color_base_finalize   (FoPropertyBackgroundColorClass *klass);
static void fo_property_background_color_class_init   (FoPropertyBackgroundColorClass *klass);
static void fo_property_background_color_finalize     (GObject       *object);

static FoDatatype* fo_property_background_color_resolve_enum (const gchar *token,
                                                              FoContext   *context,
                                                              GError     **error);
static FoDatatype* fo_property_background_color_validate (FoDatatype *datatype,
                                                          FoContext  *context,
                                                          GError    **error);

static const gchar class_name[] = "background-color";
static gpointer parent_class;
static FoDatatype *enum_transparent;

/**
 * fo_property_background_color_get_type:
 * 
 * Register the #FoPropertyBackgroundColor type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertyBackgroundColor.
 **/
GType
fo_property_background_color_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyBackgroundColorClass),
        (GBaseInitFunc) fo_property_background_color_base_init,
        (GBaseFinalizeFunc) fo_property_background_color_base_finalize,
        (GClassInitFunc) fo_property_background_color_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyBackgroundColor),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_background_color_init,
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info,
					    G_TYPE_FLAG_ABSTRACT);
    }

  return object_type;
}

/**
 * fo_property_background_color_init:
 * @background_color: #FoPropertyBackgroundColor object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertyBackgroundColor.
 **/
void
fo_property_background_color_init (FoPropertyBackgroundColor *background_color)
{
  FO_PROPERTY (background_color)->value =
    NULL;
}

/**
 * fo_property_background_color_base_init:
 * @klass: #FoPropertyBackgroundColorClass object to initialise.
 * 
 * Implements #GBaseInitFunc for #FoPropertyBackgroundColorClass.
 **/
void
fo_property_background_color_base_init (FoPropertyBackgroundColorClass *klass)
{
  enum_transparent =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_BACKGROUND_COLOR),
                  "value",
                  FO_ENUM_BACKGROUND_COLOR_TRANSPARENT,
                  NULL);
}

/**
 * fo_property_background_color_base_finalize:
 * @klass: #FoPropertyBackgroundColorClass object to finalise.
 * 
 * Implements #GBaseFinalizeFunc for #FoPropertyBackgroundColorClass.
 **/
void
fo_property_background_color_base_finalize (FoPropertyBackgroundColorClass *klass)
{
  g_object_unref (enum_transparent);
}

/**
 * fo_property_background_color_class_init:
 * @klass: #FoPropertyBackgroundColorClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertyBackgroundColorClass.
 **/
void
fo_property_background_color_class_init (FoPropertyBackgroundColorClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_background_color_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_background_color_resolve_enum;
  property_class->validate =
    fo_property_background_color_validate;
  property_class->get_initial =
    fo_property_background_color_get_initial;
}

/**
 * fo_property_background_color_finalize:
 * @object: #FoPropertyBackgroundColor object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertyBackgroundColor.
 **/
void
fo_property_background_color_finalize (GObject *object)
{
  FoPropertyBackgroundColor *background_color;

  background_color = FO_PROPERTY_BACKGROUND_COLOR (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_background_color_new:
 * 
 * Creates a new #FoPropertyBackgroundColor initialized to default value.
 * 
 * Return value: the new #FoPropertyBackgroundColor.
 **/
FoProperty*
fo_property_background_color_new (void)
{
  FoProperty* background_color;

  background_color =
    FO_PROPERTY (g_object_new (fo_property_background_color_get_type (),
                               NULL));

  return background_color;
}

/**
 * fo_property_background_color_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_background_color_resolve_enum (const gchar *token,
                                           FoContext   *context,
                                           GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "transparent") == 0)
    {
      return g_object_ref (enum_transparent);
    }
  else if (strcmp (token, "black") == 0)
    {
      return g_object_ref (fo_color_get_color_black ());
    }
  else if (strcmp (token, "silver") == 0)
    {
      return g_object_ref (fo_color_get_color_silver ());
    }
  else if (strcmp (token, "gray") == 0)
    {
      return g_object_ref (fo_color_get_color_gray ());
    }
  else if (strcmp (token, "white") == 0)
    {
      return g_object_ref (fo_color_get_color_white ());
    }
  else if (strcmp (token, "maroon") == 0)
    {
      return g_object_ref (fo_color_get_color_maroon ());
    }
  else if (strcmp (token, "red") == 0)
    {
      return g_object_ref (fo_color_get_color_red ());
    }
  else if (strcmp (token, "purple") == 0)
    {
      return g_object_ref (fo_color_get_color_purple ());
    }
  else if (strcmp (token, "fuchsia") == 0)
    {
      return g_object_ref (fo_color_get_color_fuchsia ());
    }
  else if (strcmp (token, "green") == 0)
    {
      return g_object_ref (fo_color_get_color_green ());
    }
  else if (strcmp (token, "lime") == 0)
    {
      return g_object_ref (fo_color_get_color_lime ());
    }
  else if (strcmp (token, "olive") == 0)
    {
      return g_object_ref (fo_color_get_color_olive ());
    }
  else if (strcmp (token, "yellow") == 0)
    {
      return g_object_ref (fo_color_get_color_yellow ());
    }
  else if (strcmp (token, "navy") == 0)
    {
      return g_object_ref (fo_color_get_color_navy ());
    }
  else if (strcmp (token, "blue") == 0)
    {
      return g_object_ref (fo_color_get_color_blue ());
    }
  else if (strcmp (token, "teal") == 0)
    {
      return g_object_ref (fo_color_get_color_teal ());
    }
  else if (strcmp (token, "aqua") == 0)
    {
      return g_object_ref (fo_color_get_color_aqua ());
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_background_color (context));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_background_color_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_background_color_validate (FoDatatype *datatype,
                                       FoContext  *context,
                                       GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_COLOR (datatype))
    {
      return datatype;
    }
  else if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_BACKGROUND_COLOR) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  gchar *datatype_sprintf = fo_object_sprintf (datatype);

	  g_set_error (error,
		       FO_FO_ERROR,
		       FO_FO_ERROR_DATATYPE,
		       _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		       class_name,
		       datatype_sprintf,
		       g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

	  g_object_unref (datatype);

	  g_free (datatype_sprintf);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_background_color_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_background_color_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

FoDatatype*
fo_property_background_color_new_from_expression (const gchar *expression,
					   FoContext *context,
					   FoProperty *current_font_size,
					   FoFo *fo_node,
					   GError **error)
{
  FoDatatype *datatype;
  GError *tmp_error = NULL;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);

  /*
  background_color = fo_property_background_color_new ();
  */
  datatype =
    fo_property_eval_expression (expression,
				 "background-color",
				 context,
				 fo_property_background_color_resolve_enum,
				 current_font_size,
				 fo_node);

  datatype = fo_property_background_color_validate (datatype,
						    context,
						    &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  /*
  fo_property_background_color_set_value (background_color, datatype);
  return background_color;
  */
  return datatype;
}

/**
 * fo_property_background_color_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Returns an #FoDatatype as part of ongoing test to see if FoProperty
 * child types can be abstract or can be omitted.
 *
 * Return value: An #FoDatatype.
 **/
FoDatatype*
fo_property_background_color_get_initial (void)
{
  static FoDatatype *transparent = NULL;

  if (!transparent)
    {
      transparent =
	g_object_new (FO_TYPE_ENUM,
		      "enum-class",
		      g_type_class_ref (FO_TYPE_ENUM_BACKGROUND_COLOR),
		      "value",
		      FO_ENUM_BACKGROUND_COLOR_TRANSPARENT,
		      NULL);
    }
  /*
  FoProperty *background_color;

  background_color = fo_property_background_color_new ();

  fo_property_background_color_set_value (background_color,
					  g_object_ref (enum_transparent));

  return background_color;
  */
  return transparent;
}

/**
 * fo_property_background_color_new_attr:
 * @datatype: #FoDatatype from which to create a #PangoAttribute.
 * 
 * Makes a new #PangoAttribute representing the current property.
 * 
 * Return value: New #PangoAttribute.
 **/
PangoAttribute*
fo_property_background_color_new_attr (FoDatatype *datatype)
{
  guint16 background_red;
  guint16 background_green;
  guint16 background_blue;

  g_return_val_if_fail (FO_IS_COLOR (datatype), NULL);

  background_red =
    fo_color_get_red (datatype);
  background_green =
    fo_color_get_green (datatype);
  background_blue =
    fo_color_get_blue (datatype);

  return pango_attr_background_new (background_red,
				    background_green,
				    background_blue);
}
