/* Fo
 * fo-property-margin.c: margin property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-margin.c,v 1.3 2003/03/21 14:41:38 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include "fo-property-margin-private.h"
#include "fo-property-space-before.h"
#include "fo-property-space-after.h"
#include "fo-property-space-start.h"
#include "fo-property-space-end.h"
#include "fo-property-margin-left.h"
#include "fo-property-margin-right.h"
#include "fo-property-margin-top.h"
#include "fo-property-margin-bottom.h"

/* Inherited: FALSE */
/* Shorthand: TRUE */
/* margin-left */
/* <margin-width>{1,4} | inherit */
GType
fo_enum_margin_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_MARGIN_AUTO, "FO_ENUM_MARGIN_AUTO", "auto" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumMargin", values);
    }
  return etype;
}

static void fo_property_margin_init         (FoPropertyMargin      *property_margin);
static void fo_property_margin_base_init   (FoPropertyMarginClass *klass);
static void fo_property_margin_base_finalize   (FoPropertyMarginClass *klass);
static void fo_property_margin_class_init   (FoPropertyMarginClass *klass);
static void fo_property_margin_finalize     (GObject       *object);

FoDatatype* fo_property_margin_copy(FoDatatype *datatype);
static FoDatatype* fo_property_margin_validate (FoDatatype *datatype,
						FoContext *context);
static gchar*      fo_property_margin_sprintf (FoObject *object);

static const gchar class_name[] = "margin";
static gpointer parent_class;
static FoDatatype *enum_auto;

/**
 * fo_property_margin_get_type:
 * 
 * Register the #FoPropertyMargin type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertyMargin.
 **/
GType
fo_property_margin_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyMarginClass),
        (GBaseInitFunc) fo_property_margin_base_init,
        (GBaseFinalizeFunc) fo_property_margin_base_finalize,
        (GClassInitFunc) fo_property_margin_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyMargin),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_margin_init,
	NULL		/* value_table */
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
					    class_name,
                                            &object_info, 0);
    }
  
  return object_type;
}

/**
 * fo_property_margin_init:
 * @margin: #FoPropertyMargin object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertyMargin.
 **/
void
fo_property_margin_init (FoPropertyMargin *margin)
{
  FO_PROPERTY (margin)->value = NULL;
}

/**
 * fo_property_margin_base_init:
 * @klass: #FoPropertyMarginClass object to initialise.
 * 
 * Implements #GBaseInitFunc for #FoPropertyMarginClass.
 **/
void
fo_property_margin_base_init (FoPropertyMarginClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_MARGIN),
                  "value",
                  FO_ENUM_MARGIN_AUTO,
                  NULL);
}

/**
 * fo_property_margin_base_finalize:
 * @klass: #FoPropertyMarginClass object to finalise.
 * 
 * Implements #GBaseFinalizeFunc for #FoPropertyMarginClass.
 **/
void
fo_property_margin_base_finalize (FoPropertyMarginClass *klass)
{
  g_object_unref (enum_auto);
}

/**
 * fo_property_border_class_init:
 * @klass: #FoPropertyBorderClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertyBorderClass.
 **/
void
fo_property_margin_class_init (FoPropertyMarginClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_margin_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = TRUE;
  property_class->resolve_enum =
    fo_property_margin_resolve_enum;
  property_class->validate =
    fo_property_margin_validate;
  property_class->get_initial =
    fo_property_margin_get_initial;
}

/**
 * fo_property_border_finalize:
 * @object: #FoPropertyBorder object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertyBorder.
 **/
void
fo_property_margin_finalize (GObject *object)
{
  FoPropertyMargin *margin;

  margin = FO_PROPERTY_MARGIN (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_margin_new:
 * 
 * Creates a new #FoPropertyMargin initialized to default value.
 * 
 * Return value: the new #FoPropertyMargin.
 **/
FoProperty*
fo_property_margin_new (void)
{
  FoProperty* margin;

  margin =
    FO_PROPERTY (g_object_new (fo_property_margin_get_type (),
			       NULL));
  
  return margin;
}

/**
 * fo_property_margin_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_margin_resolve_enum (const gchar *token,
				 FoContext   *context,
				 GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   "margin",
		   token);
      return NULL;
    }
}

/**
 * fo_property_margin_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_margin_validate (FoDatatype *datatype, FoContext *context)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_margin_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_margin_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else if (FO_IS_PERCENTAGE (datatype))
    {
      return datatype;
    }
  else
    {
      g_warning("Invalid margin datatype: %s (%s)",
	        fo_object_sprintf (FO_OBJECT (datatype)),
		g_type_name(G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

void
fo_property_margin_resolve (FoContext *current_context,
			    FoContext *parent_context)
{
  FoProperty *absolute, *relative;
  FoDatatype *datatype;

  g_return_if_fail (current_context != NULL);
  g_return_if_fail (parent_context != NULL);
  g_return_if_fail (FO_IS_CONTEXT (current_context));
  g_return_if_fail (FO_IS_CONTEXT (parent_context));

  /* left = start*/
  absolute = fo_context_get_margin_left (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_get_value (absolute);
      relative = fo_property_space_start_new ();
      fo_property_space_start_set_value (relative, datatype);
      fo_context_set_space_start (current_context, relative);
    }
  else
    {
      relative = fo_context_get_space_start (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_space_start_get_value (relative);
	  absolute = fo_property_margin_left_new ();
	  fo_property_set_value (absolute, datatype);
	  fo_context_set_margin_left (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_margin_left_new ();
	  fo_context_set_margin_left (current_context, absolute);
	  relative = fo_property_space_start_get_initial ();
	  fo_context_set_space_start (current_context, relative);
	}
    }
  /* right = end*/
  absolute = fo_context_get_margin_right (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_margin_right_get_value (absolute);
      relative = fo_property_space_end_new ();
      fo_property_space_end_set_value (relative, datatype);
      fo_context_set_space_end (current_context, relative);
    }
  else
    {
      relative = fo_context_get_space_end (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_space_end_get_value (relative);
	  absolute = fo_property_margin_right_new ();
	  fo_property_margin_right_set_value (absolute, datatype);
	  fo_context_set_margin_right (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_margin_right_get_initial ();
	  fo_context_set_margin_right (current_context, absolute);
	  relative = fo_property_space_end_get_initial ();
	  fo_context_set_space_end (current_context, relative);
	}
    }
  /* top = before*/
  absolute = fo_context_get_margin_top (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_margin_top_get_value (absolute);
      relative = fo_property_space_before_new ();
      fo_property_space_before_set_value (relative, datatype);
      fo_context_set_space_before (current_context, relative);
    }
  else
    {
      relative = fo_context_get_space_before (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_space_before_get_value (relative);
	  absolute = fo_property_margin_top_new ();
	  fo_property_margin_top_set_value (absolute, datatype);
	  fo_context_set_margin_top (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_margin_top_get_initial ();
	  fo_context_set_margin_top (current_context, absolute);
	  relative = fo_property_space_before_get_initial ();
	  fo_context_set_space_before (current_context, relative);
	}
    }
  /* bottom = after*/
  absolute = fo_context_get_margin_bottom (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_margin_bottom_get_value (absolute);
      relative = fo_property_space_after_new ();
      fo_property_space_after_set_value (relative, datatype);
      fo_context_set_space_after (current_context, relative);
    }
  else
    {
      relative = fo_context_get_space_after (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_space_after_get_value (relative);
	  absolute = fo_property_margin_bottom_new ();
	  fo_property_margin_bottom_set_value (absolute, datatype);
	  fo_context_set_margin_bottom (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_margin_bottom_get_initial ();
	  fo_context_set_margin_bottom (current_context, absolute);
	  relative = fo_property_space_after_get_initial ();
	  fo_context_set_space_after (current_context, relative);
	}
    }
}


/**
 * fo_property_margin_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_margin_get_initial (void)
{
  static FoProperty *margin = NULL;

  if (margin == NULL)
    {
      margin = fo_property_margin_new ();
    }

  return margin;
}

/**
 * fo_property_margin_validate_single_margin:
 * @datatype: FoDatatype to be validated against allowed datatypes and
 *            values for current property
 * @context:  FoContext object from which to possibly inherit values
 * @error:    Information about an error that has occurred
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed
 * 
 * Return value: Valid datatype value or NULL
 **/
FoDatatype*
fo_property_margin_validate_single_margin (FoDatatype *datatype,
					   FoContext  *context,
					   GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_MARGIN) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  gchar *datatype_sprintf = fo_object_sprintf (datatype);

	  g_set_error (error,
		       FO_FO_ERROR,
		       FO_FO_ERROR_DATATYPE,
		       _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		       class_name,
		       datatype_sprintf,
		       g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

	  g_object_unref (datatype);

	  g_free (datatype_sprintf);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_margin_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_margin_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else if (FO_IS_PERCENTAGE (datatype))
    {
      return datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}
