/* Fo
 * fo-property-padding.c: padding property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-padding.c,v 1.3 2003/05/16 10:55:06 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include <fo-datatype.h>
#include <fo-property.h>
#include <fo-property-padding-private.h>
#include <fo-property-padding-left.h>
#include <fo-property-padding-right.h>
#include <fo-property-padding-top.h>
#include <fo-property-padding-bottom.h>
#include <fo-property-padding-before.h>
#include <fo-property-padding-after.h>
#include <fo-property-padding-start.h>
#include <fo-property-padding-end.h>

static void fo_property_padding_init         (FoPropertyPadding      *property_padding);
static void fo_property_padding_class_init   (FoPropertyPaddingClass *klass);
static void fo_property_padding_base_class_init   (FoPropertyPaddingClass *klass);
static void fo_property_padding_base_class_finalize (FoPropertyPaddingClass *klass);
static void fo_property_padding_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_property_padding_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_property_padding_finalize     (GObject       *object);

static gpointer parent_class;

static FoDatatype *padding_initial;

GType
fo_property_padding_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyPaddingClass),
        (GBaseInitFunc) fo_property_padding_base_class_init,
        (GBaseFinalizeFunc) fo_property_padding_base_class_finalize,
        (GClassInitFunc) fo_property_padding_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyPadding),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_padding_init,
	NULL		/* value_table */
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "FoPropertyPadding",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_property_padding_init (FoPropertyPadding *padding)
{
  padding->value = NULL;
}

static void
fo_property_padding_base_class_init (FoPropertyPaddingClass *klass)
{
  padding_initial =
    g_object_new (FO_TYPE_LENGTH,
		  "value",
		  0.0,
		  NULL);
}

static void
fo_property_padding_base_class_finalize (FoPropertyPaddingClass *klass)
{
  g_object_unref (padding_initial);
}

static void
fo_property_padding_class_init (FoPropertyPaddingClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_padding_finalize;

  object_class->set_property = fo_property_padding_set_property;
  object_class->get_property = fo_property_padding_get_property;
}

static void
fo_property_padding_finalize (GObject *object)
{
  FoPropertyPadding *padding;

  padding = FO_PROPERTY_PADDING (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_property_padding_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoPropertyPadding *padding;

  padding = FO_PROPERTY_PADDING (object);

  switch (prop_id)
    {
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_property_padding_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoPropertyPadding *padding;

  padding = FO_PROPERTY_PADDING (object);

  switch (prop_id)
    {
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_padding_new:
 * 
 * Creates a new #FoPropertyPadding initialized to default value.
 * 
 * Return value: the new #FoPropertyPadding
 **/
FoProperty*
fo_property_padding_new (void)
{
  FoProperty* padding;

  padding =
    FO_PROPERTY (g_object_new (fo_property_padding_get_type (), NULL));
  
  return padding;
}

void
fo_property_padding_resolve (FoContext *current_context,
			    FoContext *parent_context)
{
  FoProperty *absolute, *relative;
  FoDatatype *datatype;

  g_return_if_fail (current_context != NULL);
  g_return_if_fail (parent_context != NULL);
  g_return_if_fail (FO_IS_CONTEXT (current_context));
  g_return_if_fail (FO_IS_CONTEXT (parent_context));

  /* left = start*/
  absolute = fo_context_get_padding_left (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_get_value (absolute);
      relative = fo_property_padding_start_new ();
      fo_property_set_value (relative, datatype);
      fo_context_set_padding_start (current_context, relative);
    }
  else
    {
      relative = fo_context_get_padding_start (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_get_value (relative);
	  absolute = fo_property_padding_left_new ();
	  fo_property_set_value (absolute, datatype);
	  fo_context_set_padding_left (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_padding_left_get_initial ();
	  fo_context_set_padding_left (current_context, absolute);
	  relative = fo_property_padding_start_get_initial ();
	  fo_context_set_padding_start (current_context, relative);
	}
    }
  /* right = end*/
  absolute = fo_context_get_padding_right (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_get_value (absolute);
      relative = fo_property_padding_end_new ();
      fo_property_set_value (relative, datatype);
      fo_context_set_padding_end (current_context, relative);
    }
  else
    {
      relative = fo_context_get_padding_end (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_get_value (relative);
	  absolute = fo_property_padding_right_new ();
	  fo_property_set_value (absolute, datatype);
	  fo_context_set_padding_right (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_padding_right_get_initial ();
	  fo_context_set_padding_right (current_context, absolute);
	  relative = fo_property_padding_end_get_initial ();
	  fo_context_set_padding_end (current_context, relative);
	}
    }
  /* top = before*/
  absolute = fo_context_get_padding_top (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_get_value (absolute);
      relative = fo_property_padding_before_new ();
      fo_property_set_value (relative, datatype);
      fo_context_set_padding_before (current_context, relative);
    }
  else
    {
      relative = fo_context_get_padding_before (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_get_value (relative);
	  absolute = fo_property_padding_top_new ();
	  fo_property_set_value (absolute, datatype);
	  fo_context_set_padding_top (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_padding_top_get_initial ();
	  fo_context_set_padding_top (current_context, absolute);
	  relative = fo_property_padding_before_get_initial ();
	  fo_context_set_padding_before (current_context, relative);
	}
    }
  /* bottom = after*/
  absolute = fo_context_get_padding_bottom (current_context);
  if (absolute != NULL)
    {
      datatype = fo_property_get_value (absolute);
      relative = fo_property_padding_after_new ();
      fo_property_set_value (relative, datatype);
      fo_context_set_padding_after (current_context, relative);
    }
  else
    {
      relative = fo_context_get_padding_after (current_context);
      if (relative != NULL)
	{
	  datatype = fo_property_get_value (relative);
	  absolute = fo_property_padding_bottom_new ();
	  fo_property_set_value (absolute, datatype);
	  fo_context_set_padding_bottom (current_context, absolute);
	}
      else
	{
	  absolute = fo_property_padding_bottom_get_initial ();
	  fo_context_set_padding_bottom (current_context, absolute);
	  relative = fo_property_padding_after_get_initial ();
	  fo_context_set_padding_after (current_context, relative);
	}
    }
}

/* padding: initial */
FoDatatype*
fo_property_padding_get_initial (void)
{
  return padding_initial;
}

