/* Fo
 * fo-property-scaling-method.c: 'scaling-method' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-scaling-method.c,v 1.2 2003/03/21 14:41:39 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include "fo-property-scaling-method.h"

/* scaling-method */
/* auto | integer-pixels | resample-any-method | inherit */
GType
fo_enum_scaling_method_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_SCALING_METHOD_AUTO, "FO_ENUM_SCALING_METHOD_AUTO", "auto" },
        { FO_ENUM_SCALING_METHOD_INTEGER_PIXELS, "FO_ENUM_SCALING_METHOD_INTEGER_PIXELS", "integer-pixels" },
        { FO_ENUM_SCALING_METHOD_RESAMPLE_ANY_METHOD, "FO_ENUM_SCALING_METHOD_RESAMPLE_ANY_METHOD", "resample-any-method" },
        { FO_ENUM_SCALING_METHOD_INHERIT, "FO_ENUM_SCALING_METHOD_INHERIT", "inherit" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumScalingMethod", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyScalingMethod
{
  FoProperty parent_instance;
};

struct _FoPropertyScalingMethodClass
{
  FoPropertyClass parent_class;
};

static void fo_property_scaling_method_init         (FoPropertyScalingMethod      *property_scaling_method);
static void fo_property_scaling_method_base_class_init   (FoPropertyScalingMethodClass *klass);
static void fo_property_scaling_method_base_class_finalize   (FoPropertyScalingMethodClass *klass);
static void fo_property_scaling_method_class_init   (FoPropertyScalingMethodClass *klass);
static void fo_property_scaling_method_set_property (GObject       *object,
                                                     guint          prop_id,
                                                     const GValue  *value,
                                                     GParamSpec    *pspec);
static void fo_property_scaling_method_get_property (GObject       *object,
                                                     guint          prop_id,
                                                     GValue        *value,
                                                     GParamSpec    *pspec);
static void fo_property_scaling_method_finalize     (GObject       *object);

static FoDatatype* fo_property_scaling_method_resolve_enum (const gchar *token,
                                                            FoContext   *context,
                                                            GError     **error);
static FoDatatype* fo_property_scaling_method_validate (FoDatatype *datatype,
                                                        FoContext  *context,
                                                        GError    **error);
static gchar*      fo_property_scaling_method_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_auto;
static FoDatatype *enum_integer_pixels;
static FoDatatype *enum_resample_any_method;

/**
 * fo_property_scaling_method_get_type:
 * 
 * Register the FoPropertyScalingMethod type if not already registered and
 * return its GType value
 * 
 * Return value: GType of FoPropertyScalingMethod
 **/
GType
fo_property_scaling_method_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyScalingMethodClass),
        (GBaseInitFunc) fo_property_scaling_method_base_class_init,
        (GBaseFinalizeFunc) fo_property_scaling_method_base_class_finalize,
        (GClassInitFunc) fo_property_scaling_method_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyScalingMethod),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_scaling_method_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "scaling-method",
                                            &object_info, 0);
    }
  
  return object_type;
}

/**
 * fo_property_scaling_method_init:
 * @scaling_method: FoPropertyScalingMethod object to initialise
 * 
 * Implements GInstanceInitFunc for FoPropertyScalingMethod
 **/
void
fo_property_scaling_method_init (FoPropertyScalingMethod *scaling_method)
{
}

/**
 * fo_property_scaling_method_base_class_init:
 * @klass: FoPropertyScalingMethodClass object to initialise
 * 
 * Implements GBaseInitFunc for FoPropertyScalingMethodClass
 **/
void
fo_property_scaling_method_base_class_init (FoPropertyScalingMethodClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SCALING_METHOD),
                  "value",
                  FO_ENUM_SCALING_METHOD_AUTO,
                  NULL);
  enum_integer_pixels =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SCALING_METHOD),
                  "value",
                  FO_ENUM_SCALING_METHOD_INTEGER_PIXELS,
                  NULL);
  enum_resample_any_method =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SCALING_METHOD),
                  "value",
                  FO_ENUM_SCALING_METHOD_RESAMPLE_ANY_METHOD,
                  NULL);
}

/**
 * fo_property_scaling_method_base_class_finalize:
 * @klass: FoPropertyScalingMethodClass object to finalise
 * 
 * Implements GBaseFinalizeFunc for FoPropertyScalingMethodClass
 **/
void
fo_property_scaling_method_base_class_finalize (FoPropertyScalingMethodClass *klass)
{
  g_object_unref (enum_auto);
  g_object_unref (enum_integer_pixels);
  g_object_unref (enum_resample_any_method);
}

/**
 * fo_property_scaling_method_class_init:
 * @klass: FoPropertyScalingMethodClass object to initialise
 * 
 * Implements GClassInitFunc for FoPropertyScalingMethodClass
 **/
void
fo_property_scaling_method_class_init (FoPropertyScalingMethodClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_scaling_method_finalize;

  object_class->set_property = fo_property_scaling_method_set_property;
  object_class->get_property = fo_property_scaling_method_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						        _("scaling-method value"),
						        _("scaling-method property value"),
							FO_TYPE_DATATYPE,
						        G_PARAM_READWRITE));

  FO_PROPERTY_CLASS (klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS (klass)->new_from_expression =
    fo_property_scaling_method_new_from_expression;
  FO_OBJECT_CLASS (klass)->sprintf =
    fo_property_scaling_method_sprintf;
  FO_PROPERTY_CLASS (klass)->set_value =
    fo_property_scaling_method_set_value;
  FO_PROPERTY_CLASS (klass)->get_value =
    fo_property_scaling_method_get_value;
  FO_PROPERTY_CLASS(klass)->resolve_enum =
    fo_property_scaling_method_resolve_enum;
  FO_PROPERTY_CLASS(klass)->validate =
    fo_property_scaling_method_validate;
}

/**
 * fo_property_scaling_method_finalize:
 * @object: FoPropertyScalingMethod object to finalize
 * 
 * Implements GObjectFinalizeFunc for FoPropertyScalingMethod
 **/
void
fo_property_scaling_method_finalize (GObject *object)
{
  FoPropertyScalingMethod *scaling_method;

  scaling_method = FO_PROPERTY_SCALING_METHOD (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_scaling_method_set_property:
 * @object:  GObject whose property will be set
 * @prop_id: Property ID assigned when property registered
 * @value:   New value for property
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectSetPropertyFunc for FoScalingMethod
 **/
void
fo_property_scaling_method_set_property (GObject      *object,
                                         guint         prop_id,
                                         const GValue *value,
                                         GParamSpec   *pspec)
{
  FoPropertyScalingMethod *scaling_method;

  scaling_method = FO_PROPERTY_SCALING_METHOD (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_scaling_method_set_value (FO_PROPERTY (scaling_method), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_scaling_method_get_property:
 * @object:  GObject whose property will be retreived
 * @prop_id: Property ID assigned when property registered
 * @value:   GValue to set with property value
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectGetPropertyFunc for FoScalingMethod
 **/
void
fo_property_scaling_method_get_property (GObject    *object,
                                         guint       prop_id,
                                         GValue     *value,
                                         GParamSpec *pspec)
{
  FoPropertyScalingMethod *scaling_method;

  scaling_method = FO_PROPERTY_SCALING_METHOD (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_scaling_method_get_value (FO_PROPERTY (scaling_method)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_scaling_method_new:
 * 
 * Creates a new #FoPropertyScalingMethod initialized to default value.
 * 
 * Return value: the new #FoPropertyScalingMethod
 **/
FoProperty*
fo_property_scaling_method_new (void)
{
  FoProperty* scaling_method;

  scaling_method =
    FO_PROPERTY (g_object_new (fo_property_scaling_method_get_type (), NULL));
  
  return scaling_method;
}

/**
 * fo_property_scaling_method_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *            enumeration token
 * @context: FoContext object from which to possibly inherit values
 * @error:   Information about an error that has occurred
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL
 * 
 * Return value: Resolved enumeration value or NULL
 **/
FoDatatype*
fo_property_scaling_method_resolve_enum (const gchar *token,
                                         FoContext   *context,
                                         GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "integer-pixels") == 0)
    {
      return g_object_ref (enum_integer_pixels);
    }
  else if (strcmp (token, "resample-any-method") == 0)
    {
      return g_object_ref (enum_resample_any_method);
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_scaling_method (context));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   "scaling-method",
		   token);
      return NULL;
    }
}

/**
 * fo_property_scaling_method_validate:
 * @datatype: FoDatatype to be validated against allowed datatypes and
 *            values for current property
 * @context:  FoContext object from which to possibly inherit values
 * @error:    Information about an error that has occurred
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed
 * 
 * Return value: Valid datatype value or NULL
 **/
FoDatatype*
fo_property_scaling_method_validate (FoDatatype *datatype,
                                     FoContext  *context,
                                     GError    **error)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_SCALING_METHOD) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_scaling_method_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_scaling_method_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   "scaling-method",
		   fo_object_sprintf (datatype),
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

/**
 * fo_property_scaling_method_new_from_expression:
 * @expression:        Expression to evaluate
 * @context:           FoContext object from which to possibly inherit values
 * @current_font_size: Current font size
 * @fo_node:           Formatting object on which property expression was
 *                     declared
 * @error:             Information about an error that has occurred
 * 
 * Evaluates @expression w.r.t. @context and @current_font_size.  Returns
 * an FoProperty object of correct type if expression evaluated without
 * error, or NULL if an error occurred.
 * 
 * Return value: FoProperty object representing resolved expression, or NULL
 **/
FoProperty*
fo_property_scaling_method_new_from_expression (const gchar *expression,
                                                FoContext   *context,
                                                FoProperty  *current_font_size,
                                                FoFo        *fo_node,
                                                GError     **error)
{
  FoProperty *scaling_method;
  FoDatatype *datatype;
  GError *tmp_error = NULL;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  datatype =
    fo_property_eval_expression2 (expression,
                                  "scaling-method",
				  context,
				  fo_property_scaling_method_resolve_enum,
				  current_font_size,
				  fo_node,
                                  &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  datatype = fo_property_scaling_method_validate (datatype, context, &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  scaling_method = fo_property_scaling_method_new ();

  fo_property_scaling_method_set_value (scaling_method, datatype);

  return scaling_method;
}

/**
 * fo_property_scaling_method_set_value:
 * @property:  FoProperty object whose value is to be set
 * @new_value: New value for the property
 * 
 * Sets the #value property of @property
 **/
void
fo_property_scaling_method_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  g_return_if_fail (property != NULL);
  g_return_if_fail (FO_IS_PROPERTY_SCALING_METHOD (property));

  if (new_value)
    g_object_ref (G_OBJECT (new_value));
  if (property->value)
    g_object_unref (G_OBJECT (property->value));
  property->value = new_value;
  /*g_object_notify (G_OBJECT(scaling_method), "value");*/
}

/**
 * fo_property_scaling_method_get_value:
 * @property: FoProperty object whose value is to be retrieved
 * 
 * Returns the current #value property value of @property
 * 
 * Return value: The current #value property value
 **/
FoDatatype*
fo_property_scaling_method_get_value (FoProperty *property)
{
  g_return_val_if_fail (property != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_SCALING_METHOD (property), NULL);

  return property->value;
}

/**
 * fo_property_scaling_method_sprintf:
 * @object: Object to be named
 * 
 * Returns a string representing the property and its current value.
 * The returned value should be freed when no longer needed.
 *
 * Return value: A newly allocated string
 **/
gchar*
fo_property_scaling_method_sprintf (FoObject *object)
{
  g_return_val_if_fail (object != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_SCALING_METHOD (object), NULL);

  return
    g_strdup_printf ("scaling-method: %s",
		     fo_object_sprintf (FO_PROPERTY (object)->value));
}

/**
 * fo_property_scaling_method_get_initial:
 * 
 * Returns an instance of the property with the correct initial value
 * 
 * Return value: An instance of the property
 **/
FoProperty*
fo_property_scaling_method_get_initial (void)
{
  static FoProperty *scaling_method = NULL;

  if (!scaling_method)
    {
      scaling_method = fo_property_scaling_method_new ();

      /* Set the initial value here */
      fo_property_scaling_method_set_value (scaling_method,
                 NULL);
    }

  return scaling_method;
}
