/* Fo
 * fo-property-scaling.c: 'scaling' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-scaling.c,v 1.2 2003/03/21 14:41:40 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include "fo-property-scaling.h"

/* scaling */
/* uniform | non-uniform | inherit */
GType
fo_enum_scaling_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_SCALING_UNIFORM, "FO_ENUM_SCALING_UNIFORM", "uniform" },
        { FO_ENUM_SCALING_NON_UNIFORM, "FO_ENUM_SCALING_NON_UNIFORM", "non-uniform" },
        { FO_ENUM_SCALING_INHERIT, "FO_ENUM_SCALING_INHERIT", "inherit" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumScaling", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyScaling
{
  FoProperty parent_instance;
};

struct _FoPropertyScalingClass
{
  FoPropertyClass parent_class;
};

static void fo_property_scaling_init         (FoPropertyScaling      *property_scaling);
static void fo_property_scaling_base_class_init   (FoPropertyScalingClass *klass);
static void fo_property_scaling_base_class_finalize   (FoPropertyScalingClass *klass);
static void fo_property_scaling_class_init   (FoPropertyScalingClass *klass);
static void fo_property_scaling_set_property (GObject       *object,
                                              guint          prop_id,
                                              const GValue  *value,
                                              GParamSpec    *pspec);
static void fo_property_scaling_get_property (GObject       *object,
                                              guint          prop_id,
                                              GValue        *value,
                                              GParamSpec    *pspec);
static void fo_property_scaling_finalize     (GObject       *object);

static FoDatatype* fo_property_scaling_resolve_enum (const gchar *token,
                                                     FoContext   *context,
                                                     GError     **error);
static FoDatatype* fo_property_scaling_validate (FoDatatype *datatype,
                                                 FoContext  *context,
                                                 GError    **error);
static gchar*      fo_property_scaling_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_uniform;
static FoDatatype *enum_non_uniform;

/**
 * fo_property_scaling_get_type:
 * 
 * Register the FoPropertyScaling type if not already registered and
 * return its GType value
 * 
 * Return value: GType of FoPropertyScaling
 **/
GType
fo_property_scaling_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyScalingClass),
        (GBaseInitFunc) fo_property_scaling_base_class_init,
        (GBaseFinalizeFunc) fo_property_scaling_base_class_finalize,
        (GClassInitFunc) fo_property_scaling_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyScaling),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_scaling_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "scaling",
                                            &object_info, 0);
    }
  
  return object_type;
}

/**
 * fo_property_scaling_init:
 * @scaling: FoPropertyScaling object to initialise
 * 
 * Implements GInstanceInitFunc for FoPropertyScaling
 **/
void
fo_property_scaling_init (FoPropertyScaling *scaling)
{
}

/**
 * fo_property_scaling_base_class_init:
 * @klass: FoPropertyScalingClass object to initialise
 * 
 * Implements GBaseInitFunc for FoPropertyScalingClass
 **/
void
fo_property_scaling_base_class_init (FoPropertyScalingClass *klass)
{
  enum_uniform =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SCALING),
                  "value",
                  FO_ENUM_SCALING_UNIFORM,
                  NULL);
  enum_non_uniform =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SCALING),
                  "value",
                  FO_ENUM_SCALING_NON_UNIFORM,
                  NULL);
}

/**
 * fo_property_scaling_base_class_finalize:
 * @klass: FoPropertyScalingClass object to finalise
 * 
 * Implements GBaseFinalizeFunc for FoPropertyScalingClass
 **/
void
fo_property_scaling_base_class_finalize (FoPropertyScalingClass *klass)
{
  g_object_unref (enum_uniform);
  g_object_unref (enum_non_uniform);
}

/**
 * fo_property_scaling_class_init:
 * @klass: FoPropertyScalingClass object to initialise
 * 
 * Implements GClassInitFunc for FoPropertyScalingClass
 **/
void
fo_property_scaling_class_init (FoPropertyScalingClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_scaling_finalize;

  object_class->set_property = fo_property_scaling_set_property;
  object_class->get_property = fo_property_scaling_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						        _("scaling value"),
						        _("scaling property value"),
							FO_TYPE_DATATYPE,
						        G_PARAM_READWRITE));

  FO_PROPERTY_CLASS (klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS (klass)->new_from_expression =
    fo_property_scaling_new_from_expression;
  FO_OBJECT_CLASS (klass)->sprintf =
    fo_property_scaling_sprintf;
  FO_PROPERTY_CLASS (klass)->set_value =
    fo_property_scaling_set_value;
  FO_PROPERTY_CLASS (klass)->get_value =
    fo_property_scaling_get_value;
  FO_PROPERTY_CLASS(klass)->resolve_enum =
    fo_property_scaling_resolve_enum;
  FO_PROPERTY_CLASS(klass)->validate =
    fo_property_scaling_validate;
}

/**
 * fo_property_scaling_finalize:
 * @object: FoPropertyScaling object to finalize
 * 
 * Implements GObjectFinalizeFunc for FoPropertyScaling
 **/
void
fo_property_scaling_finalize (GObject *object)
{
  FoPropertyScaling *scaling;

  scaling = FO_PROPERTY_SCALING (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_scaling_set_property:
 * @object:  GObject whose property will be set
 * @prop_id: Property ID assigned when property registered
 * @value:   New value for property
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectSetPropertyFunc for FoScaling
 **/
void
fo_property_scaling_set_property (GObject      *object,
                                  guint         prop_id,
                                  const GValue *value,
                                  GParamSpec   *pspec)
{
  FoPropertyScaling *scaling;

  scaling = FO_PROPERTY_SCALING (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_scaling_set_value (FO_PROPERTY (scaling), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_scaling_get_property:
 * @object:  GObject whose property will be retreived
 * @prop_id: Property ID assigned when property registered
 * @value:   GValue to set with property value
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectGetPropertyFunc for FoScaling
 **/
void
fo_property_scaling_get_property (GObject    *object,
                                  guint       prop_id,
                                  GValue     *value,
                                  GParamSpec *pspec)
{
  FoPropertyScaling *scaling;

  scaling = FO_PROPERTY_SCALING (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_scaling_get_value (FO_PROPERTY (scaling)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_scaling_new:
 * 
 * Creates a new #FoPropertyScaling initialized to default value.
 * 
 * Return value: the new #FoPropertyScaling
 **/
FoProperty*
fo_property_scaling_new (void)
{
  FoProperty* scaling;

  scaling =
    FO_PROPERTY (g_object_new (fo_property_scaling_get_type (), NULL));
  
  return scaling;
}

/**
 * fo_property_scaling_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *            enumeration token
 * @context: FoContext object from which to possibly inherit values
 * @error:   Information about an error that has occurred
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL
 * 
 * Return value: Resolved enumeration value or NULL
 **/
FoDatatype*
fo_property_scaling_resolve_enum (const gchar *token,
                                  FoContext   *context,
                                  GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "uniform") == 0)
    {
      return g_object_ref (enum_uniform);
    }
  else if (strcmp (token, "non-uniform") == 0)
    {
      return g_object_ref (enum_non_uniform);
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_scaling (context));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   "scaling",
		   token);
      return NULL;
    }
}

/**
 * fo_property_scaling_validate:
 * @datatype: FoDatatype to be validated against allowed datatypes and
 *            values for current property
 * @context:  FoContext object from which to possibly inherit values
 * @error:    Information about an error that has occurred
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed
 * 
 * Return value: Valid datatype value or NULL
 **/
FoDatatype*
fo_property_scaling_validate (FoDatatype *datatype,
                              FoContext  *context,
                              GError    **error)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_SCALING) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_scaling_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_scaling_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   "scaling",
		   fo_object_sprintf (datatype),
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

/**
 * fo_property_scaling_new_from_expression:
 * @expression:        Expression to evaluate
 * @context:           FoContext object from which to possibly inherit values
 * @current_font_size: Current font size
 * @fo_node:           Formatting object on which property expression was
 *                     declared
 * @error:             Information about an error that has occurred
 * 
 * Evaluates @expression w.r.t. @context and @current_font_size.  Returns
 * an FoProperty object of correct type if expression evaluated without
 * error, or NULL if an error occurred.
 * 
 * Return value: FoProperty object representing resolved expression, or NULL
 **/
FoProperty*
fo_property_scaling_new_from_expression (const gchar *expression,
                                         FoContext   *context,
                                         FoProperty  *current_font_size,
                                         FoFo        *fo_node,
                                         GError     **error)
{
  FoProperty *scaling;
  FoDatatype *datatype;
  GError *tmp_error = NULL;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  datatype =
    fo_property_eval_expression2 (expression,
                                  "scaling",
				  context,
				  fo_property_scaling_resolve_enum,
				  current_font_size,
				  fo_node,
                                  &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  datatype = fo_property_scaling_validate (datatype, context, &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  scaling = fo_property_scaling_new ();

  fo_property_scaling_set_value (scaling, datatype);

  return scaling;
}

/**
 * fo_property_scaling_set_value:
 * @property:  FoProperty object whose value is to be set
 * @new_value: New value for the property
 * 
 * Sets the #value property of @property
 **/
void
fo_property_scaling_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  g_return_if_fail (property != NULL);
  g_return_if_fail (FO_IS_PROPERTY_SCALING (property));

  if (new_value)
    g_object_ref (G_OBJECT (new_value));
  if (property->value)
    g_object_unref (G_OBJECT (property->value));
  property->value = new_value;
  /*g_object_notify (G_OBJECT(scaling), "value");*/
}

/**
 * fo_property_scaling_get_value:
 * @property: FoProperty object whose value is to be retrieved
 * 
 * Returns the current #value property value of @property
 * 
 * Return value: The current #value property value
 **/
FoDatatype*
fo_property_scaling_get_value (FoProperty *property)
{
  g_return_val_if_fail (property != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_SCALING (property), NULL);

  return property->value;
}

/**
 * fo_property_scaling_sprintf:
 * @object: Object to be named
 * 
 * Returns a string representing the property and its current value.
 * The returned value should be freed when no longer needed.
 *
 * Return value: A newly allocated string
 **/
gchar*
fo_property_scaling_sprintf (FoObject *object)
{
  g_return_val_if_fail (object != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_SCALING (object), NULL);

  return
    g_strdup_printf ("scaling: %s",
		     fo_object_sprintf (FO_PROPERTY (object)->value));
}

/**
 * fo_property_scaling_get_initial:
 * 
 * Returns an instance of the property with the correct initial value
 * 
 * Return value: An instance of the property
 **/
FoProperty*
fo_property_scaling_get_initial (void)
{
  static FoProperty *scaling = NULL;

  if (!scaling)
    {
      scaling = fo_property_scaling_new ();

      /* Set the initial value here */
      fo_property_scaling_set_value (scaling,
                 NULL);
    }

  return scaling;
}
