/* Fo
 * fo-property-table-layout.c: table-layout property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-table-layout.c,v 1.2 2003/03/21 14:41:45 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include "fo-property-table-layout.h"

/* table-layout */
/* auto | fixed | inherit */
GType
fo_enum_table_layout_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_TABLE_LAYOUT_AUTO, "FO_ENUM_TABLE_LAYOUT_AUTO", "auto" },
        { FO_ENUM_TABLE_LAYOUT_FIXED, "FO_ENUM_TABLE_LAYOUT_FIXED", "fixed" },
        { FO_ENUM_TABLE_LAYOUT_INHERIT, "FO_ENUM_TABLE_LAYOUT_INHERIT", "inherit" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumTableLayout", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyTableLayout
{
  FoProperty parent_instance;

  FoDatatype *value;
};

struct _FoPropertyTableLayoutClass
{
  FoPropertyClass parent_class;
  
};

static void fo_property_table_layout_init         (FoPropertyTableLayout      *property_table_layout);
static void fo_property_table_layout_base_class_init   (FoPropertyTableLayoutClass *klass);
static void fo_property_table_layout_base_class_finalize   (FoPropertyTableLayoutClass *klass);
static void fo_property_table_layout_class_init   (FoPropertyTableLayoutClass *klass);
static void fo_property_table_layout_set_property (GObject       *object,
                                                   guint          prop_id,
                                                   const GValue  *value,
                                                   GParamSpec    *pspec);
static void fo_property_table_layout_get_property (GObject       *object,
                                                   guint          prop_id,
                                                   GValue        *value,
                                                   GParamSpec    *pspec);
static void fo_property_table_layout_finalize     (GObject       *object);

FoDatatype* fo_property_table_layout_copy (FoDatatype *datatype);
static FoDatatype* fo_property_table_layout_resolve_enum (const gchar *token,
                                                          FoContext   *context,
                                                          GError     **error);
static FoDatatype* fo_property_table_layout_validate (FoDatatype *datatype,
                                                      FoContext  *context,
                                                      GError    **error);
static gchar*      fo_property_table_layout_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_auto;
static FoDatatype *enum_fixed;

/**
 * fo_property_table_layout_get_type:
 * 
 * Register the FoPropertyTableLayout type if not already registered and
 * return its GType value
 * 
 * Return value: GType of FoPropertyTableLayout
 **/
GType
fo_property_table_layout_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyTableLayoutClass),
        (GBaseInitFunc) fo_property_table_layout_base_class_init,
        (GBaseFinalizeFunc) fo_property_table_layout_base_class_finalize,
        (GClassInitFunc) fo_property_table_layout_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyTableLayout),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_table_layout_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "FoPropertyTableLayout",
                                            &object_info, 0);
    }
  
  return object_type;
}

/**
 * fo_property_table_layout_init:
 * @table_layout: FoPropertyTableLayout object to initialise
 * 
 * Implements GInstanceInitFunc for FoPropertyTableLayout
 **/
void
fo_property_table_layout_init (FoPropertyTableLayout *table_layout)
{
  table_layout->value = NULL;
}

/**
 * fo_property_table_layout_base_class_init:
 * @klass: FoPropertyTableLayoutClass object to initialise
 * 
 * Implements GBaseInitFunc for FoPropertyTableLayoutClass
 **/
void
fo_property_table_layout_base_class_init (FoPropertyTableLayoutClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_TABLE_LAYOUT),
                  "value",
                  FO_ENUM_TABLE_LAYOUT_AUTO,
                  NULL);
  enum_fixed =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_TABLE_LAYOUT),
                  "value",
                  FO_ENUM_TABLE_LAYOUT_FIXED,
                  NULL);
}

/**
 * fo_property_table_layout_base_class_finalize:
 * @klass: FoPropertyTableLayoutClass object to finalise
 * 
 * Implements GBaseFinalizeFunc for FoPropertyTableLayoutClass
 **/
void
fo_property_table_layout_base_class_finalize (FoPropertyTableLayoutClass *klass)
{
  g_object_unref (enum_auto);
  g_object_unref (enum_fixed);
}

/**
 * fo_property_table_layout_class_init:
 * @klass: FoPropertyTableLayoutClass object to initialise
 * 
 * Implements GClassInitFunc for FoPropertyTableLayoutClass
 **/
void
fo_property_table_layout_class_init (FoPropertyTableLayoutClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_table_layout_finalize;

  object_class->set_property = fo_property_table_layout_set_property;
  object_class->get_property = fo_property_table_layout_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						        _("table-layout value"),
						        _("table-layout property value"),
							FO_TYPE_DATATYPE,
						        G_PARAM_READWRITE));

  FO_PROPERTY_CLASS (klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS (klass)->new_from_expression =
    fo_property_table_layout_new_from_expression;
  FO_OBJECT_CLASS (klass)->sprintf =
    fo_property_table_layout_sprintf;
  FO_PROPERTY_CLASS (klass)->set_value =
    fo_property_table_layout_set_value;
  FO_PROPERTY_CLASS (klass)->get_value =
    fo_property_table_layout_get_value;
  FO_PROPERTY_CLASS(klass)->resolve_enum =
    fo_property_table_layout_resolve_enum;
  FO_PROPERTY_CLASS(klass)->validate =
    fo_property_table_layout_validate;
}

/**
 * fo_property_table_layout_finalize:
 * @object: FoPropertyTableLayout object to finalize
 * 
 * Implements GObjectFinalizeFunc for FoPropertyTableLayout
 **/
void
fo_property_table_layout_finalize (GObject *object)
{
  FoPropertyTableLayout *table_layout;

  table_layout = FO_PROPERTY_TABLE_LAYOUT (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_table_layout_set_property:
 * @object:  GObject whose property will be set
 * @prop_id: Property ID assigned when property registered
 * @value:   New value for property
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectSetPropertyFunc for FoTableLayout
 **/
void
fo_property_table_layout_set_property (GObject      *object,
                                       guint         prop_id,
                                       const GValue *value,
                                       GParamSpec   *pspec)
{
  FoPropertyTableLayout *table_layout;

  table_layout = FO_PROPERTY_TABLE_LAYOUT (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_table_layout_set_value (FO_PROPERTY (table_layout), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_table_layout_get_property:
 * @object:  GObject whose property will be retreived
 * @prop_id: Property ID assigned when property registered
 * @value:   GValue to set with property value
 * @pspec:   Parameter specification for this property type
 * 
 * Implements #GObjectGetPropertyFunc for FoTableLayout
 **/
void
fo_property_table_layout_get_property (GObject    *object,
                                       guint       prop_id,
                                       GValue     *value,
                                       GParamSpec *pspec)
{
  FoPropertyTableLayout *table_layout;

  table_layout = FO_PROPERTY_TABLE_LAYOUT (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_table_layout_get_value (FO_PROPERTY (table_layout)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_table_layout_new:
 * 
 * Creates a new #FoPropertyTableLayout initialized to default value.
 * 
 * Return value: the new #FoPropertyTableLayout
 **/
FoProperty*
fo_property_table_layout_new (void)
{
  FoProperty* table_layout;

  table_layout =
    FO_PROPERTY (g_object_new (fo_property_table_layout_get_type (), NULL));
  
  return table_layout;
}

/**
 * fo_property_table_layout_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *            enumeration token
 * @context: FoContext object from which to possibly inherit values
 * @error:   Information about an error that has occurred
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL
 * 
 * Return value: Resolved enumeration value or NULL
 **/
FoDatatype*
fo_property_table_layout_resolve_enum (const gchar *token,
                                       FoContext   *context,
                                       GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "fixed") == 0)
    {
      return g_object_ref (enum_fixed);
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_table_layout (context));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   "table-layout",
		   token);
      return NULL;
    }
}

/**
 * fo_property_table_layout_validate:
 * @datatype: FoDatatype to be validated against allowed datatypes and
 *            values for current property
 * @context:  FoContext object from which to possibly inherit values
 * @error:    Information about an error that has occurred
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed
 * 
 * Return value: Valid datatype value or NULL
 **/
FoDatatype*
fo_property_table_layout_validate (FoDatatype *datatype,
                                   FoContext  *context,
                                   GError    **error)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_TABLE_LAYOUT) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_table_layout_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_table_layout_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   "table-layout",
		   fo_object_sprintf (FO_OBJECT (datatype)),
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

/**
 * fo_property_table_layout_new_from_expression:
 * @expression:        Expression to evaluate
 * @context:           FoContext object from which to possibly inherit values
 * @current_font_size: Current font size
 * @fo_node:           Formatting object on which property expression was
 *                     declared
 * @error:             Information about an error that has occurred
 * 
 * Evaluates @expression w.r.t. @context and @current_font_size.  Returns
 * an FoProperty object of correct type if expression evaluated without
 * error, or NULL if an error occurred.
 * 
 * Return value: FoProperty object representing resolved expression, or NULL
 **/
FoProperty*
fo_property_table_layout_new_from_expression (const gchar *expression,
                                              FoContext   *context,
                                              FoProperty  *current_font_size,
                                              FoFo        *fo_node,
                                              GError     **error)
{
  FoProperty *table_layout;
  FoDatatype *datatype;
  GError *tmp_error = NULL;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  datatype =
    fo_property_eval_expression2 (expression,
                                  "table-layout",
				  context,
				  fo_property_table_layout_resolve_enum,
				  current_font_size,
				  fo_node,
                                  &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  datatype = fo_property_table_layout_validate (datatype, context, &tmp_error);

  if (tmp_error != NULL)
    {
      g_propagate_error (error, tmp_error);
      return NULL;
    }

  table_layout = fo_property_table_layout_new ();

  fo_property_table_layout_set_value (table_layout, datatype);

  return table_layout;
}

/**
 * fo_property_table_layout_set_value:
 * @property:  FoProperty object whose value is to be set
 * @new_value: New value for the property
 * 
 * Sets the #value property of @property
 **/
void
fo_property_table_layout_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  FoPropertyTableLayout *table_layout;

  g_return_if_fail (property != NULL);
  g_return_if_fail (FO_IS_PROPERTY_TABLE_LAYOUT (property));

  table_layout = FO_PROPERTY_TABLE_LAYOUT (property);

  table_layout->value = new_value;
  /*g_object_notify (G_OBJECT(table_layout), "value");*/
}

/**
 * fo_property_table_layout_get_value:
 * @property: FoProperty object whose value is to be retrieved
 * 
 * Returns the current #value property value of @property
 * 
 * Return value: The current #value property value
 **/
FoDatatype*
fo_property_table_layout_get_value (FoProperty *property)
{
  FoPropertyTableLayout *table_layout;

  g_return_val_if_fail (property != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_TABLE_LAYOUT (property), NULL);

  table_layout = FO_PROPERTY_TABLE_LAYOUT (property);

  return table_layout->value;
}

/**
 * fo_property_table_layout_sprintf:
 * @object: Object to be named
 * 
 * Returns a string representing the property and its current value.
 * The returned value should be freed when no longer needed.
 *
 * Return value: A newly allocated string
 **/
gchar*
fo_property_table_layout_sprintf (FoObject *object)
{
  FoDatatype *datatype;

  g_return_val_if_fail (object != NULL, NULL);
  g_return_val_if_fail (FO_IS_PROPERTY_TABLE_LAYOUT (object), NULL);

  datatype = fo_property_table_layout_get_value (FO_PROPERTY (object));

  return
    g_strdup_printf ("table-layout: %s",
		     fo_object_sprintf (FO_OBJECT (datatype)));
}

/**
 * fo_property_table_layout_get_initial:
 * 
 * Returns an instance of the property with the correct initial value
 * 
 * Return value: An instance of the property
 **/
FoProperty*
fo_property_table_layout_get_initial (void)
{
  static FoProperty *table_layout = NULL;

  if (!table_layout)
    {
      table_layout = fo_property_table_layout_new ();

      /* Set the initial value here */
      fo_property_table_layout_set_value (table_layout,
					  enum_auto);
    }

  return table_layout;
}
